﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Security;
using System.Reflection;
using System.Diagnostics;
using System.Threading;

namespace viperPS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            /* If application was started with a command-line argument, update a visual cue for the user. */
            bool restartImpersonation = false;
            string[] arguments = Environment.GetCommandLineArgs();
            foreach (string arg in arguments.Skip(1))
            {
                if (arg.ToLower().Trim() == "i")
                {
                    restartImpersonation = true;
                }
            }
            if (restartImpersonation == true)
            {
                txtBoxNotification.Text = ">> Running under impersonation context.";
            }
        }

        #region Helper Methods
        /// <summary>
        /// Clear output text boxes in preparation of data being inserted into them.
        /// </summary>
        private void clearOutputTxtBoxes()
        {
            txtBoxResponseOutput.Text = "Working, please wait...";
            txtBoxErrorOutput.Text = "";
        }

        /// <summary>
        /// Accepts the List array returned by the PS methods and puts the results into the response text boxes.
        /// </summary>
        /// <param name="actionResult"></param>
        private void HandlePSResponseToOutputBoxes(List<KeyValuePair<string, string>> actionResult)
        {
            string responseData = String.Empty;
            string responseError = String.Empty;
            foreach (KeyValuePair<string, string> keyValuePairs in actionResult)
            {
                if (keyValuePairs.Key.ToLower() == "psdata") { responseData = keyValuePairs.Value; }
                if (keyValuePairs.Key.ToLower() == "error") { responseError = keyValuePairs.Value; }
            }
            /* Dump data into the output textboxes */
            txtBoxResponseOutput.Text = responseData;
            txtBoxErrorOutput.Text = responseError;
        }
        #endregion

        /// <summary>
        /// Locks target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLockMachine_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSLockMachine(tmpUserTargetComputer));
        }

        /// <summary>
        /// Logs off user of target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLogOffUser_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSLogOffUser(tmpUserTargetComputer));
        }

        /// <summary>
        /// Restarts target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRestartMachine_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSRestartMachine(tmpUserTargetComputer));
        }

        /// <summary>
        /// Get the SID of the logged in user.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetUserSID_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetUserSID(tmpUserTargetComputer));
        }

        /// <summary>
        /// Get disk space information of target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetDiskspace_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetDiskspaceMachine(tmpUserTargetComputer));
        }

        /// <summary>
        /// Get hostname of target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetMachineName_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetMachineName(tmpUserTargetComputer));
        }
        
        /// <summary>
        /// Get IP address(s) of target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetMachineIP_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetMachineIP(tmpUserTargetComputer));
        }

        /// <summary>
        /// Perform ICMP Ping to target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPingMachine_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSPingMachine(tmpUserTargetComputer));
        }

        /// <summary>
        /// Restart this application using a different account.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnImpersonate_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            StringBuilder psResult = new StringBuilder();
            string psErrorData = String.Empty;

            /* Get required information for restart */
            string tmpU = txtBoxConnectU.Text.ToString().Trim();
            txtBoxConnectU.Text = "";

            if (tmpU.Length > 0)
            {
                /* The following is what you use if you needed to transform regular text to a secure string, such as from a textbox */
                //SecureString sUP = new SecureString();
                //foreach (char c in tmpP.ToCharArray())
                //{
                //    sUP.AppendChar(c);
                //}
                //sUP.MakeReadOnly();
                //tmpP = String.Empty;

                /* Get domain or workstation equivalent if not in a domain and no domain was specified */
                string tmpD = String.Empty;
                if (tmpU.IndexOf("\\") > -1)
                {
                    tmpD = tmpU.Split('\\')[0];
                    tmpU = tmpU.Split('\\')[1];
                }
                else
                {
                    /* Assume domain user may be logged into, if in a domain and, if not, the computer name in a non-domain */
                    List<KeyValuePair<string, string>> actionResult = PSWork.PSGetLoggedInUser("::1");
                    string responseData = String.Empty;
                    string responseError = String.Empty;
                    foreach (KeyValuePair<string, string> keyValuePairs in actionResult)
                    {
                        if (keyValuePairs.Key.ToLower() == "psdata") { responseData = keyValuePairs.Value; }
                        if (keyValuePairs.Key.ToLower() == "error") { responseData = keyValuePairs.Value; }
                    }
                    if (responseData.Length > 0)
                    {
                        tmpD = responseData.Split('\\')[0];
                    }
                }

                if (tmpD.Length > 0)
                {
                    Process currentProcess = Process.GetCurrentProcess();
                    int currentAppProcessID = currentProcess.Id;
                    var fullPath = Assembly.GetEntryAssembly().Location;
                    /* Process will fail if credentials are not valid */
                    try
                    {
                        Process.Start(fullPath, "i", tmpU, PwdUserControl1.WpfSecurePassword, tmpD);
                    }
                    catch (Exception ex)
                    {
                        psErrorData = "[ERROR] " + ex.ToString();
                    }

                    if (psErrorData.Length == 0)
                    {
                        psResult.Append("Refreshing with impersonation.");
                        var responseList = new List<KeyValuePair<string, string>>()
                        {
                            new KeyValuePair<string, string>("psdata", psResult.ToString()),
                            new KeyValuePair<string, string>("error", psErrorData)
                        };
                        HandlePSResponseToOutputBoxes(responseList);
                        Application.Exit();
                    }
                    else
                    {
                        /* Show credential problem */
                        var responseList = new List<KeyValuePair<string, string>>()
                        {
                            new KeyValuePair<string, string>("psdata", psResult.ToString()),
                            new KeyValuePair<string, string>("error", psErrorData)
                        };
                        HandlePSResponseToOutputBoxes(responseList);
                    }
                }
                else
                {
                    /* Domain not provided and could not be determined */
                    psErrorData = "[ERROR] Domain or workstation equivalent could not be determined or was not provided.  You will need to manually close this application and start it with \"Run As\".";
                    var responseList = new List<KeyValuePair<string, string>>()
                    {
                        new KeyValuePair<string, string>("psdata", psResult.ToString()),
                        new KeyValuePair<string, string>("error", psErrorData)
                    };
                    HandlePSResponseToOutputBoxes(responseList);
                }
            }
            else
            {
                psErrorData = "[ERROR] Credentials need to be entered for impersonation.";
                var responseList = new List<KeyValuePair<string, string>>()
                    {
                        new KeyValuePair<string, string>("psdata", psResult.ToString()),
                        new KeyValuePair<string, string>("error", psErrorData)
                    };
                HandlePSResponseToOutputBoxes(responseList);
            }
        }

        /// <summary>
        /// Get connection and hardware information of the target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetMachineInfo_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetMachineInfo(tmpUserTargetComputer));
        }

        /// <summary>
        /// Get processes of user on target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetMachineProcesses_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            string tmpUserTargetUser = txtBoxTargetUser.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetMachineProcesses(tmpUserTargetComputer, tmpUserTargetUser));
        }

        /// <summary>
        /// Perform Trace route to target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTraceRouteMachine_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSTraceRouteMachine(tmpUserTargetComputer));
        }

        /// <summary>
        /// Get the logged in user of the target machine.  If target machine is not specified, assume target is local machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetLoggedInUser_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetLoggedInUser(tmpUserTargetComputer));
        }

        /// <summary>
        /// Get uptime of target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetMachineUptime_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetMachineUptime(tmpUserTargetComputer));
        }

        /// <summary>
        /// Starts remote assistance to target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteAssistanceToMachine_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSRemoteAssistanceToMachine(tmpUserTargetComputer));
        }

        /// <summary>
        /// Starts a RDP session with target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteToMachine_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSRemoteToMachine(tmpUserTargetComputer));
        }

        /// <summary>
        /// Open the computer management pane on the target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMachineManagement_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSMachineManagement(tmpUserTargetComputer));
        }

        /// <summary>
        /// Clears the CSC offline files cache on target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCscOfflineFilesClean_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSCscOfflineFilesClean(tmpUserTargetComputer));
        }

        /// <summary>
        /// Starts a command-linen or powershell window to allow commands to be manually entered on the target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRunRemoteCommand_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            string tmpcomboBoxRunRemoteCommand = comboBoxRunRemoteCommand.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSRunRemoteCommand(tmpUserTargetComputer, tmpcomboBoxRunRemoteCommand));
        }

        /// <summary>
        /// Get machine log from target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetMachineEventLogs_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            string tmpcomboBoxMachineEventLog = comboBoxMachineEventLog.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetMachineEventLogs(tmpUserTargetComputer, tmpcomboBoxMachineEventLog));
        }

        /// <summary>
        /// Get services and state from target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetServices_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetServices(tmpUserTargetComputer));
        }

        /// <summary>
        /// Restart a specific service on the target machine of a user.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRestartService_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            string tmpTargetService = txtBoxRestartService.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSRestartService(tmpUserTargetComputer, tmpTargetService));
        }

        /// <summary>
        /// Provides browse window dialog to allow user to select a text file containing computer names - one name per line
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBrowseFileMultipleComputerList_Click(object sender, EventArgs e)
        {
            OpenFileDialog openComputerListDialog = new OpenFileDialog
            {
                Title = "Browse to TEXT file containing list of computer names - one per line",
                Filter = "Text Files (*.txt)|*.txt",
                FilterIndex = 1,
                ReadOnlyChecked = true,
                ShowReadOnly = true
            };
            /* Get filename from user's interaction of the file dialog window */
            if (openComputerListDialog.ShowDialog() == DialogResult.OK)
            {
                /* Put the path in the text box as visual cue the file was selected */
                txtBoxBrowseComputerList.Text = openComputerListDialog.FileName;
            }
        }

        /// <summary>
        /// Provides browse window dialog to allow user to select a bat or ps1 file on the local computer that will be executed on the target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBrowseBatPsFileMultiple_Click(object sender, EventArgs e)
        {
            OpenFileDialog openBatPsFileMultipleDialog = new OpenFileDialog
            {
                Title = "Browse to BAT or PS1 file to run on Target Machine/IPs",
                Filter = "Batch Files (*.bat)|*.bat|PS Files (*.ps1)|*.ps1",
                FilterIndex = 1,
                ReadOnlyChecked = true,
                ShowReadOnly = true
            };
            /* Get filename from user's interaction of the file dialog window */
            if (openBatPsFileMultipleDialog.ShowDialog() == DialogResult.OK)
            {
                /* Put the path in the text box as visual cue the file was selected */
                txtBoxBrowseBatPs1Multiple.Text = openBatPsFileMultipleDialog.FileName;
            }
        }

        /// <summary>
        /// Accepts a local file of target computers and a different local file to run on multiple target machines and runs it on the target computers.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRunBatPs1Multiple_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get computer list of target computers and local file to run on them */
            string tmpUserTargetComputers = txtBoxBrowseComputerList.Text.ToString().Trim();
            string tmpUserLocalFileMultiple = txtBoxBrowseBatPs1Multiple.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSRunBatPsTargetMachines(tmpUserTargetComputers, tmpUserLocalFileMultiple));
        }

        /// <summary>
        /// Provides browse window dialog to allow user to select a bat or ps1 file on the local machine that will be executed on the target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBrowseBatPsFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog openBatPsFileDialog = new OpenFileDialog
            {
                Title = "Browse to BAT or PS1 file to run on Target Machine/IP",
                Filter = "Batch Files (*.bat)|*.bat|PS Files (*.ps1)|*.ps1",
                FilterIndex = 1,
                ReadOnlyChecked = true,
                ShowReadOnly = true
            };
            /* Get filename from user's interaction of the file dialog window */
            if (openBatPsFileDialog.ShowDialog() == DialogResult.OK)
            {
                /* Put the path in the textbox as visual cue the file was selected */
                txtBoxBrowseBatPs1.Text = openBatPsFileDialog.FileName;
            }
        }

        /// <summary>
        /// Accepts the local file to run on the target machine and runs it on the target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRunBatPs1_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            string tmpUserLocalFile = txtBoxBrowseBatPs1.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSRunBatPsTargetMachine(tmpUserTargetComputer, tmpUserLocalFile));
        }

        /// <summary>
        /// Determines powershell version on the target machine.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPSVersion_Click(object sender, EventArgs e)
        {
            clearOutputTxtBoxes();
            /* Get Target ComputerName if the user entered it */
            string tmpUserTargetComputer = txtBoxTargetComputerIP.Text.ToString().Trim();
            /* Complete PS Request */
            HandlePSResponseToOutputBoxes(PSWork.PSGetVersion(tmpUserTargetComputer));
        }

        /// <summary>
        /// Copies content in the request output to the clipboard without forcing the application into a single thread.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCopyOutput_Click(object sender, EventArgs e)
        {
            string tmpBoxContent = txtBoxResponseOutput.Text.ToString().Trim();
            Thread STAThread = new Thread(delegate () { System.Windows.Forms.Clipboard.SetText(tmpBoxContent); });
            STAThread.SetApartmentState(ApartmentState.STA);
            STAThread.Start();
            STAThread.Join();
        }

        /// <summary>
        /// Copies content in the request error output to the clipboard without forcing the application into a single thread.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCopyErrorOutput_Click(object sender, EventArgs e)
        {
            string tmpBoxContent = txtBoxErrorOutput.Text.ToString().Trim();
            Thread STAThread = new Thread(delegate () { System.Windows.Forms.Clipboard.SetText(tmpBoxContent); });
            STAThread.SetApartmentState(ApartmentState.STA);
            STAThread.Start();
            STAThread.Join();
        }

        /// <summary>
        /// Calls the About form pane.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAbout_Click(object sender, EventArgs e)
        {
            Form2 About = new Form2();
            About.Show();
        }
    }
}
